
#ifndef Sconn_includeD
#define Sconn_includeD Yes

#define Sconnfd_linesizE 65336
#define Sconnfd_writebufmaX (256*1024)

/* usually  : 60 */
#define Sconn_handshake_timeouT 4000

#define Sconnfd_min_writeratE 4096.0

#define Sconnfd_typelisT \
                        "file","file","tcp_accept","tcp","none","unknown"

struct SconnfD {

 int references; 

 int type;     /* 0= classical file (usually a named pipe)
                  1= temporary file (usually a named pipe), remove on close
                  2= TCP/IP master socket (for accept)
                  3= TCP/IP data socket
                  4= dummy device, do not open, select, read, write, close
                  5= borrowed fd of unknown type: handle as file, do not close
                */

 int fd;
 struct SconnfD *borrowed_from; /* ==NULL : original holder of fd */    

 char *filename;

 char *buffer;
 int bufflimit;
 int buffsize;
 int buffcount;
 double min_writerate;

 int eof_reached;

};


struct SconN {

 char *id;

 int references; 

 struct SaccesS *access;
 int tunnel_required;
 int tunnel_allowed;
 int tunnel_expect_zero;
 char *tunnel_read_buf;
 char *tunnel_read_pt;
 int tunnel_read_count;
 struct StunneL *tunnel;
 int tunnel_failed;


 struct SconnfD *in_cfd;

 struct SconnfD *out_cfd;

};


/*


A SconN object implements a communication connection which may be either
unidirectional or bidirectional. Bidirectional connections may consist
of two independent unidirectional channels (i.e. pipes). Usually bidirectional 
communication is handled asynchronously.
Each channel is represented by a SconnfD object.

Transmissions may be authenticated and obscured via a StunneL object.
Usually this object is created during a short synchronous handshake at
the very start of communication. To control the use of this tunneling
mechanism, each SconN may be linked to a SaccesS object which contains
usernames and access rules. 

Currently SconN communication channels are restricted to file descriptors.
In most cases SconN will not open the descriptor by itself but will get 
it from an application which already performed open() or socket().
Nevertheless there are functions in sconn.c which do open filedescriptors.
SconN should know about the special properties of a channel (see member type
in struct SconnfD) so it can handle and close it properly.
Sconn_attach_fd() is used to make a SconnfD from an open file descriptor.


Usage of SconN:

Since a SconN may be shared by several owners, it contains a reference
counter which preserves its existence as long as one of its owners
is still alive. One owner is already counted by Sconn_new()  .
Therefore it is important to assign further SconN pointers by calls to
Sconn_refer() rather than by the operator = . Use either Sconn_destroy() or
its alias Sconn_unrefer() to release and possibly destroy an SconN.
  ... create object X ...
  Sconn_refer(sconn,&(X.sconn_pointer),0);
  ... use X and its sconn_pointer ...
  Sconn_unrefer(sconn,0);
  ... destroy X ...


Reading and Writing:
 
Reading and writing via a SconN is done by Sconn_read(), Sconn_readchar(),
Sconn_readline() and Sconn_write() which perform actions similar to read(),
getc(), fgets() and write().
I/o is nonblocking. So be aware of getting data in small pieces or even
no data at all (return value == 2). Fragmentation may occur due to tunneling
or due to the underlying transport mechanisms.
The function  Sconn_is_ready()  may be used to determine wether a channel
is likely to accept or deliver data. In rare cases a read or write may
return 2 even if a channel was seen ready shortly before.

Regarding all other specialities of particular connections, SconN should
be completely tranparent for its applicators.
Note that Sconn_readline() possibly buffers bytes of incomplete lines which
will thus not be delivered by subsequent calls of Sconn_read() or
Sconn_readchar(). The functions are syncronized only after Sconn_readline()
returned 1 (rather than 2).


Tunneling:

Tunneling is usually initiated by a call to Sconn_try_tunnel() after both
channels have been equipped with open filedescriptors. This function performs
its action only if SaccesS queries to 
  "connection",type,"user"
  "connection",type,"keydirectory"
return nonempty values which can be combined to the name of a keyfile
  ${keydirectory}/${user}.tnl
which contains enough bytes for a StunneL key (usually 16 bytes).
type is the type of the input channel (mostly "file" or "tcp").

The tunneling protocol can be chosen by 
  "connection",type,"clientprotocol"
  "connection",type,"serverprotocol"
Valid protocols are "0.0", "0.2" and for serverprotocol "all". See stunnel.h
for more about those protocols.

If on a server tunneling is allowed or required by the SaccesS value of
 "connection",type,"tunnel"
for the channel's type, then a newly created SconN automatically calls
 Sconn_expect_tunnel() on the first read attempt.
Type usually is one of:      
 "tcp","file","all","<"+filename 

If the value is "require" then a tunnel handshake has to be performed
before any data can be transmitted.
If the value is "allow" then a tunnel handshake has to be performed
before any data are transmitted. Be sure no data begins with the bytes
 "-handshake:" if you intend to use this setting.
Any other value disables the expectation of tunneling.

A good default setting for the SaccesS rules may be:
 "connection","all","tunnel","refuse"
 "connection","tcp","tunnel","allow"
 "connection","all","keydirectory",{a suitable directory with keyfiles}
 "connection","all","user",{a user name known to both sides}     
As already mentioned, the keydirectory and the username form a file address
where the user's tunneling key may be found:
  ${keydirectory}/${user}.tnl
It is important that both sides of a tunnel get the same key from their
copies of the keyfiles. Since the file content is encrypted by a key
specific to the StunneL version, the file contents may differ.
It is more easy to handle if both sides use the same version of StunneL.
(see stunnel.h for details on internal, personal and session keys.)

If not protocol 0.2 was forced by "serverprotocol" :
After Sconn_try_tunnel() has been finished successfully, the other side
still has no reason to believe in authenticity. Only after the first
data transmission could be successfully unpacked, it is clear that the
same key is used on both sides.
If no data is to be transmitted immediately after Sconn_try_tunnel() then
a call of Sconn_reauthenticate() should be made. This will cause the other
end of the tunnel to return 1 if queried by function Stunnel_is_authenticated()
but will transmit no data.


SconN as TCP/IP Client:

A standard use of SconN is client-server-communication via TCP/IP.
The function  Sconn_tcpclient_new()  creates a new client which is connected
to a certain port at a certain host. If secure communication is desired,
a call to Sconn_try_tunnel() should follow (see above: Tunneling).

A corresponding TCP/IP server will generate a socket by accept() and
attach it to a SconN by two calls of Sconn_attach_fd() . One for the
input channel and one for output. If tunneling is enabled, the SconN
object will expect a handshake as first transmission. 



*/ 


#endif /* ! Sconn_includeD */


