
/*
 cc -c shash.c
*/

#include <sys/types.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

#include "smem.h"

#include "shash.h"


/* -------------------------- MD5 ------------------------ */

/* This code is derived from RFC 1321 and implements computation of the
    "RSA Data Security, Inc. MD5 Message-Digest Algorithm" */

/* F, G, H and I are basic MD5 functions.
 */
#define Shash_md5_F(x, y, z) (((x) & (y)) | ((~x) & (z)))
#define Shash_md5_G(x, y, z) (((x) & (z)) | ((y) & (~z)))
#define Shash_md5_H(x, y, z) ((x) ^ (y) ^ (z))
#define Shash_md5_I(x, y, z) ((y) ^ ((x) | (~z)))

/* ROTATE_LEFT rotates x left n bits.
 */
#define Shash_md5_ROTATE_LEFT(x, n) (((x) << (n)) | ((x) >> (32-(n))))

/* FF, GG, HH, and II transformations for rounds 1, 2, 3, and 4.
Rotation is separate from addition to prevent recomputation.
 */
#define Shash_md5_FF(a, b, c, d, x, s, ac) { \
 (a) += Shash_md5_F ((b), (c), (d)) + (x) + (unsigned long)(ac); \
 (a) = Shash_md5_ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define Shash_md5_GG(a, b, c, d, x, s, ac) { \
 (a) += Shash_md5_G ((b), (c), (d)) + (x) + (unsigned long)(ac); \
 (a) = Shash_md5_ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define Shash_md5_HH(a, b, c, d, x, s, ac) { \
 (a) += Shash_md5_H ((b), (c), (d)) + (x) + (unsigned long)(ac); \
 (a) = Shash_md5_ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }
#define Shash_md5_II(a, b, c, d, x, s, ac) { \
 (a) += Shash_md5_I ((b), (c), (d)) + (x) + (unsigned long)(ac); \
 (a) = Shash_md5_ROTATE_LEFT ((a), (s)); \
 (a) += (b); \
  }


/* MD5 basic transformation. Transforms state based on block.
 */
int Shash_md5__transform (state, block)
unsigned long state[4];
unsigned char block[64];
{
 unsigned long a = state[0], b = state[1], c = state[2], d = state[3], x[16];
 unsigned int i, j;

 for (i = 0, j = 0; j < 64; i++, j += 4)
   x[i] = ((unsigned long)block[j]) | (((unsigned long)block[j+1]) << 8) |
    (((unsigned long)block[j+2]) << 16) | (((unsigned long)block[j+3]) << 24);

  /* Round 1 */
  Shash_md5_FF (a, b, c, d, x[ 0], Shash_md5_S11, 0xd76aa478); /* 1 */
  Shash_md5_FF (d, a, b, c, x[ 1], Shash_md5_S12, 0xe8c7b756); /* 2 */
  Shash_md5_FF (c, d, a, b, x[ 2], Shash_md5_S13, 0x242070db); /* 3 */
  Shash_md5_FF (b, c, d, a, x[ 3], Shash_md5_S14, 0xc1bdceee); /* 4 */
  Shash_md5_FF (a, b, c, d, x[ 4], Shash_md5_S11, 0xf57c0faf); /* 5 */
  Shash_md5_FF (d, a, b, c, x[ 5], Shash_md5_S12, 0x4787c62a); /* 6 */
  Shash_md5_FF (c, d, a, b, x[ 6], Shash_md5_S13, 0xa8304613); /* 7 */
  Shash_md5_FF (b, c, d, a, x[ 7], Shash_md5_S14, 0xfd469501); /* 8 */
  Shash_md5_FF (a, b, c, d, x[ 8], Shash_md5_S11, 0x698098d8); /* 9 */
  Shash_md5_FF (d, a, b, c, x[ 9], Shash_md5_S12, 0x8b44f7af); /* 10 */
  Shash_md5_FF (c, d, a, b, x[10], Shash_md5_S13, 0xffff5bb1); /* 11 */
  Shash_md5_FF (b, c, d, a, x[11], Shash_md5_S14, 0x895cd7be); /* 12 */
  Shash_md5_FF (a, b, c, d, x[12], Shash_md5_S11, 0x6b901122); /* 13 */
  Shash_md5_FF (d, a, b, c, x[13], Shash_md5_S12, 0xfd987193); /* 14 */
  Shash_md5_FF (c, d, a, b, x[14], Shash_md5_S13, 0xa679438e); /* 15 */
  Shash_md5_FF (b, c, d, a, x[15], Shash_md5_S14, 0x49b40821); /* 16 */

  /* Round 2 */
  Shash_md5_GG (a, b, c, d, x[ 1], Shash_md5_S21, 0xf61e2562); /* 17 */
  Shash_md5_GG (d, a, b, c, x[ 6], Shash_md5_S22, 0xc040b340); /* 18 */
  Shash_md5_GG (c, d, a, b, x[11], Shash_md5_S23, 0x265e5a51); /* 19 */
  Shash_md5_GG (b, c, d, a, x[ 0], Shash_md5_S24, 0xe9b6c7aa); /* 20 */
  Shash_md5_GG (a, b, c, d, x[ 5], Shash_md5_S21, 0xd62f105d); /* 21 */
  Shash_md5_GG (d, a, b, c, x[10], Shash_md5_S22,  0x2441453); /* 22 */
  Shash_md5_GG (c, d, a, b, x[15], Shash_md5_S23, 0xd8a1e681); /* 23 */
  Shash_md5_GG (b, c, d, a, x[ 4], Shash_md5_S24, 0xe7d3fbc8); /* 24 */
  Shash_md5_GG (a, b, c, d, x[ 9], Shash_md5_S21, 0x21e1cde6); /* 25 */
  Shash_md5_GG (d, a, b, c, x[14], Shash_md5_S22, 0xc33707d6); /* 26 */
  Shash_md5_GG (c, d, a, b, x[ 3], Shash_md5_S23, 0xf4d50d87); /* 27 */
  Shash_md5_GG (b, c, d, a, x[ 8], Shash_md5_S24, 0x455a14ed); /* 28 */
  Shash_md5_GG (a, b, c, d, x[13], Shash_md5_S21, 0xa9e3e905); /* 29 */
  Shash_md5_GG (d, a, b, c, x[ 2], Shash_md5_S22, 0xfcefa3f8); /* 30 */
  Shash_md5_GG (c, d, a, b, x[ 7], Shash_md5_S23, 0x676f02d9); /* 31 */
  Shash_md5_GG (b, c, d, a, x[12], Shash_md5_S24, 0x8d2a4c8a); /* 32 */

  /* Round 3 */
  Shash_md5_HH (a, b, c, d, x[ 5], Shash_md5_S31, 0xfffa3942); /* 33 */
  Shash_md5_HH (d, a, b, c, x[ 8], Shash_md5_S32, 0x8771f681); /* 34 */
  Shash_md5_HH (c, d, a, b, x[11], Shash_md5_S33, 0x6d9d6122); /* 35 */
  Shash_md5_HH (b, c, d, a, x[14], Shash_md5_S34, 0xfde5380c); /* 36 */
  Shash_md5_HH (a, b, c, d, x[ 1], Shash_md5_S31, 0xa4beea44); /* 37 */
  Shash_md5_HH (d, a, b, c, x[ 4], Shash_md5_S32, 0x4bdecfa9); /* 38 */
  Shash_md5_HH (c, d, a, b, x[ 7], Shash_md5_S33, 0xf6bb4b60); /* 39 */
  Shash_md5_HH (b, c, d, a, x[10], Shash_md5_S34, 0xbebfbc70); /* 40 */
  Shash_md5_HH (a, b, c, d, x[13], Shash_md5_S31, 0x289b7ec6); /* 41 */
  Shash_md5_HH (d, a, b, c, x[ 0], Shash_md5_S32, 0xeaa127fa); /* 42 */
  Shash_md5_HH (c, d, a, b, x[ 3], Shash_md5_S33, 0xd4ef3085); /* 43 */
  Shash_md5_HH (b, c, d, a, x[ 6], Shash_md5_S34,  0x4881d05); /* 44 */
  Shash_md5_HH (a, b, c, d, x[ 9], Shash_md5_S31, 0xd9d4d039); /* 45 */
  Shash_md5_HH (d, a, b, c, x[12], Shash_md5_S32, 0xe6db99e5); /* 46 */
  Shash_md5_HH (c, d, a, b, x[15], Shash_md5_S33, 0x1fa27cf8); /* 47 */
  Shash_md5_HH (b, c, d, a, x[ 2], Shash_md5_S34, 0xc4ac5665); /* 48 */

  /* Round 4 */
  Shash_md5_II (a, b, c, d, x[ 0], Shash_md5_S41, 0xf4292244); /* 49 */
  Shash_md5_II (d, a, b, c, x[ 7], Shash_md5_S42, 0x432aff97); /* 50 */
  Shash_md5_II (c, d, a, b, x[14], Shash_md5_S43, 0xab9423a7); /* 51 */
  Shash_md5_II (b, c, d, a, x[ 5], Shash_md5_S44, 0xfc93a039); /* 52 */
  Shash_md5_II (a, b, c, d, x[12], Shash_md5_S41, 0x655b59c3); /* 53 */
  Shash_md5_II (d, a, b, c, x[ 3], Shash_md5_S42, 0x8f0ccc92); /* 54 */
  Shash_md5_II (c, d, a, b, x[10], Shash_md5_S43, 0xffeff47d); /* 55 */
  Shash_md5_II (b, c, d, a, x[ 1], Shash_md5_S44, 0x85845dd1); /* 56 */
  Shash_md5_II (a, b, c, d, x[ 8], Shash_md5_S41, 0x6fa87e4f); /* 57 */
  Shash_md5_II (d, a, b, c, x[15], Shash_md5_S42, 0xfe2ce6e0); /* 58 */
  Shash_md5_II (c, d, a, b, x[ 6], Shash_md5_S43, 0xa3014314); /* 59 */
  Shash_md5_II (b, c, d, a, x[13], Shash_md5_S44, 0x4e0811a1); /* 60 */
  Shash_md5_II (a, b, c, d, x[ 4], Shash_md5_S41, 0xf7537e82); /* 61 */
  Shash_md5_II (d, a, b, c, x[11], Shash_md5_S42, 0xbd3af235); /* 62 */
  Shash_md5_II (c, d, a, b, x[ 2], Shash_md5_S43, 0x2ad7d2bb); /* 63 */
  Shash_md5_II (b, c, d, a, x[ 9], Shash_md5_S44, 0xeb86d391); /* 64 */

  state[0] += a;
  state[1] += b;
  state[2] += c;
  state[3] += d;

  /* Zeroize sensitive information. */
  memset ((char *) x, 0, sizeof (x));
  return(1);
}


int Shash_md5__encode(output, input, len)
unsigned char *output;
unsigned long *input;
unsigned int len;
{
  unsigned int i, j;

  for (i = 0, j = 0; j < len; i++, j += 4) {
    output[j] = (unsigned char)(input[i] & 0xff);
    output[j+1] = (unsigned char)((input[i] >> 8) & 0xff);
    output[j+2] = (unsigned char)((input[i] >> 16) & 0xff);
    output[j+3] = (unsigned char)((input[i] >> 24) & 0xff);
  }
  return(1);
}



int Shash_md5_init(ctx,flag)
struct Shash_md5_ctX *ctx;
int flag;
{
 ctx->count[0] = ctx->count[1] = 0;
 /* Load magic initialization constants. */
 ctx->state[0] = 0x67452301;
 ctx->state[1] = 0xefcdab89;
 ctx->state[2] = 0x98badcfe;
 ctx->state[3] = 0x10325476;
 return(1);
}


/* MD5 block update operation. Continues an MD5 message-digest
  operation, processing another message block, and updating the
  context.
 */
int Shash_md5_update(ctx,data,datalen,flag)
struct Shash_md5_ctX *ctx;
unsigned char *data;
int datalen;
int flag;
{
 unsigned int i, index, partlen;

 /* Compute number of bytes mod 64 */
 index = (unsigned int)((ctx->count[0] >> 3) & 0x3F);
 /* Update number of bits */
 if ((ctx->count[0] += ((unsigned long) datalen << 3)) < 
     ((unsigned long) datalen << 3))
   ctx->count[1]++;
 ctx->count[1] += ((unsigned long) datalen >> 29);
 partlen = 64 - index;

 /* Transform as many times as possible. */
 if (datalen >= partlen) {
   memcpy((char *) &ctx->buffer[index], (char *) data, partlen);
   Shash_md5__transform(ctx->state, ctx->buffer);
   for (i = partlen; i + 63 < datalen; i += 64)
     Shash_md5__transform(ctx->state, &data[i]);
   index = 0;
 } else
   i = 0;

 /* Buffer remaining data */
 memcpy((char *) &ctx->buffer[index], (char *) &data[i],datalen-i);

 return(1);
}


int Shash_md5_final(ctx,result,flag)
struct Shash_md5_ctX *ctx;
char result[16];
int flag;
{
 unsigned char bits[8];
 unsigned int index, padlen;
 static unsigned char PADDING[64] = {
   0x80, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
   0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
 };

 /* Save number of bits */
 Shash_md5__encode(bits, ctx->count, 8);
 /* Pad out to 56 mod 64.  */
 index = (unsigned int)((ctx->count[0] >> 3) & 0x3f);
 padlen = (index < 56) ? (56 - index) : (120 - index);
 Shash_md5_update(ctx, PADDING, padlen,0);
 /* Append length (before padding) */
 Shash_md5_update(ctx, bits, 8,0);
 /* Store state in result */
 Shash_md5__encode(result, ctx->state, 16);
 /* Zeroize sensitive information.  */
 memset ((char *) ctx, 0, sizeof (*ctx));
 return(1);
}


int Shash_md5(ctx,data,datalen,result,flag)
struct Shash_md5_ctX **ctx;     /* *ctx has to be NULL or point to freeable memory */
unsigned char *data;
int datalen;
char result[16];
int flag;
/*
 bit0= allocate and init *ctx
 bit1= transfer ctx to result
 bit2= with bit 0 : clone new *ctx from (struct Shash_md5_ctX *) data
 bit15= free *ctx
*/
{
 if(flag&(1<<15)) {
   if(*ctx!=NULL)
     Smem_freE((char *) *ctx);
   *ctx= NULL;
   return(1);
 }
 if(flag&1) {
   if(*ctx!=NULL)
     Smem_freE((char *) *ctx);
   *ctx= TSOB_FELD(struct Shash_md5_ctX,1);
   if(*ctx==NULL)
     return(-1);
   Shash_md5_init(*ctx,0);
   if(flag&4) {
     memcpy((char *) *ctx,data,sizeof(struct Shash_md5_ctX));
     datalen= 0;
   }
 }
 if(*ctx==NULL)
   return(0);
 if(datalen>0)
   Shash_md5_update(*ctx,data,datalen,0);
 if(flag&2) {
   Shash_md5_final(*ctx,result,0);
 }
 return(1);
}


/* -------------------------- end MD5 ------------------------ */


/* -------------------------- SHA-1 -------------------------- */

/*
 Nicely free implementation found at  http://4049.nwr.jp/comp/sha1.html 
 Quote :
"
Based upon: NIST FIPS180-1 Secure Hash Algorithm-1
  http://www.itl.nist.gov/fipspubs/fip180-1.htm
...
 Copyright (C) 2002 vi@nwr.jp. All rights reserved.

 This software is provided 'as-is', without any express or implied
 warranty. In no event will the authors be held liable for any damages
 arising from the use of this software.

 Permission is granted to anyone to use this software for any purpose,
 including commercial applications, and to alter it and redistribute it
 freely, subject to the following restrictions:

 1. The origin of this software must not be misrepresented; you must not
    claim that you wrote the original software. If you use this software
    in a product, an acknowledgement in the product documentation would be
    appreciated but is not required.
 2. Altered source versions must be plainly marked as such, and must not be
    misrepresented as beging the original software.
 3. This notice may not be removed or altered from any source distribution.
"
end quote
Plain mark : This is a version altered by Thomas Schmitt, stic-source@gmx.net
             (released under BSD open source license)
*/


/*
 * Packing bytes to a word
 *
 * Should not assume p is aligned to word boundary
 */
static Shash_sha1_inlinE Shash_sha1_worD Shash_sha1_packup(p)
Shash_sha1_bytE *p;
{
  /* Portable, but slow */
  return (p[0] << 24 | p[1] << 16 | p[2] << 8 | p[3] << 0);
}


/*
 * Unpacking a word to bytes
 *
 * Should not assume p is aligned to word boundary
 */
static void Shash_sha1_unpackup(p,q)
Shash_sha1_bytE *p;
Shash_sha1_worD q;
{
  p[0] = (q >> 24) & 0xff;
  p[1] = (q >> 16) & 0xff;
  p[2] = (q >>  8) & 0xff;
  p[3] = (q >>  0) & 0xff;
}


/*
 * Processing a block
 */
static void Shash_sha1_update_now(pms,bp)           /* was Shash_sha1_inlinE */
struct Shash_sha1_ctX *pms;
Shash_sha1_bytE *bp;
{
  Shash_sha1_worD tmp, a, b, c, d, e, w[16+16];
  int i, s;

  /* pack 64 bytes into 16 words */
  for(i = 0; i < 16; i++) {
    w[i] = Shash_sha1_packup(bp + i * sizeof(Shash_sha1_worD));
  }
  memcpy(w + 16, w + 0, sizeof(Shash_sha1_worD) * 16);

  a = pms->sha1_h[0];
  b = pms->sha1_h[1];
  c = pms->sha1_h[2];
  d = pms->sha1_h[3];
  e = pms->sha1_h[4];

#define Shash_sha1_rot(x,n) (((x) << n) | ((x) >> (32-n)))
#define Shash_sha1_f0(b, c, d) ((b&c)|(~b&d))
#define Shash_sha1_f1(b, c, d) (b^c^d)
#define Shash_sha1_f2(b, c, d) ((b&c)|(b&d)|(c&d))
#define Shash_sha1_f3(b, c, d) (b^c^d)

  /* t=0-15 */
  s = 0;
  for(i = 0; i < 16; i++) {
    tmp = Shash_sha1_rot(a, 5) + Shash_sha1_f0(b, c, d) + e + w[s] 
          + Shash_sha1_k0;
    e = d; d = c; c = Shash_sha1_rot(b, 30); b = a; a = tmp;
    s = (s + 1) % 16;
  }

  /* t=16-19 */
  for(i = 16; i < 20; i++) {
    w[s] = Shash_sha1_rot(w[s+13] ^ w[s+8] ^ w[s+2] ^ w[s], 1);
    w[s+16] = w[s];
    tmp = Shash_sha1_rot(a, 5) + Shash_sha1_f0(b, c, d) + e + w[s] 
          + Shash_sha1_k0;
    e = d; d = c; c = Shash_sha1_rot(b, 30); b = a; a = tmp;
    s = (s + 1) % 16;
  }

  /* t=20-39 */
  for(i = 0; i < 20; i++) {
    w[s] = Shash_sha1_rot(w[s+13] ^ w[s+8] ^ w[s+2] ^ w[s], 1);
    w[s+16] = w[s];
    tmp = Shash_sha1_rot(a, 5) + Shash_sha1_f1(b, c, d) + e + w[s]
          + Shash_sha1_k1;
    e = d; d = c; c = Shash_sha1_rot(b, 30); b = a; a = tmp;
    s = (s + 1) % 16;
  }

  /* t=40-59 */
  for(i = 0; i < 20; i++) {
    w[s] = Shash_sha1_rot(w[s+13] ^ w[s+8] ^ w[s+2] ^ w[s], 1);
    w[s+16] = w[s];
    tmp = Shash_sha1_rot(a, 5) + Shash_sha1_f2(b, c, d) + e + w[s]
          + Shash_sha1_k2;
    e = d; d = c; c = Shash_sha1_rot(b, 30); b = a; a = tmp;
    s = (s + 1) % 16;
  }

  /* t=60-79 */
  for(i = 0; i < 20; i++) {
    w[s] = Shash_sha1_rot(w[s+13] ^ w[s+8] ^ w[s+2] ^ w[s], 1);
    w[s+16] = w[s];
    tmp = Shash_sha1_rot(a, 5) + Shash_sha1_f3(b, c, d) + e + w[s]
          + Shash_sha1_k3;
    e = d; d = c; c = Shash_sha1_rot(b, 30); b = a; a = tmp;
    s = (s + 1) % 16;
  }

  pms->sha1_h[0] += a, pms->sha1_h[1] += b, pms->sha1_h[2] += c, 
  pms->sha1_h[3] += d, pms->sha1_h[4] += e;
}


/*
 * Increment sha1_size1, sha1_size2 field of struct Shash_sha1_ctX
 */
static Shash_sha1_inlinE void Shash_sha1_incr(pms,v)
struct Shash_sha1_ctX *pms;
int v;
{
  Shash_sha1_worD q;

  q = pms->sha1_size1 + v * Shash_sha1_bitS;
  if(q < pms->sha1_size1) {
    pms->sha1_size2++;
  }
  pms->sha1_size1 = q;
}


/*
 * Initialize struct Shash_sha1_ctX as FIPS specifies
 */
void Shash_sha1_init(pms)
struct Shash_sha1_ctX *pms;
{
  memset(pms, 0, sizeof(*pms));
  pms->sha1_h[0] = 0x67452301; /* Initialize H[0]-H[4] */
  pms->sha1_h[1] = 0xEFCDAB89;
  pms->sha1_h[2] = 0x98BADCFE;
  pms->sha1_h[3] = 0x10325476;
  pms->sha1_h[4] = 0xC3D2E1F0;
}


/*
 * Fill block and update output when needed
 */
void Shash_sha1_update(pms,bufp,length)
struct Shash_sha1_ctX *pms;
Shash_sha1_bytE *bufp;
int length;
{
  /* Is the buffer partially filled? */
  if(pms->sha1_count != 0) {
    if(pms->sha1_count + length >= sizeof(pms->sha1_buf)) { 
                                                  /* buffer is filled enough */
      int fil = sizeof(pms->sha1_buf) - pms->sha1_count;  /* length to copy */

      memcpy(pms->sha1_buf + pms->sha1_count, bufp, fil);
      Shash_sha1_update_now(pms, pms->sha1_buf);
      length -= fil;
      bufp += fil;
      pms->sha1_count = 0;
      Shash_sha1_incr(pms, fil);
    } else {
      memcpy(pms->sha1_buf + pms->sha1_count, bufp, length);
      pms->sha1_count += length;
      Shash_sha1_incr(pms, length);
      return;
    }
  }

  /* Loop to update state */
  for(;;) {
    if(length < sizeof(pms->sha1_buf)) {  /* Short to fill up the buffer */
      if(length) {
        memcpy(pms->sha1_buf, bufp, length);
      }
      pms->sha1_count = length;
      Shash_sha1_incr(pms, length);
      break;
    }
    Shash_sha1_update_now(pms, bufp);
    length -= sizeof(pms->sha1_buf);
    bufp += sizeof(pms->sha1_buf);
    Shash_sha1_incr(pms, sizeof(pms->sha1_buf));
  }
}


void Shash_sha1_finish(pms,output)
struct Shash_sha1_ctX *pms;
Shash_sha1_bytE output[Shash_sha1_output_sizE];
{
  int i;
  Shash_sha1_bytE buf[1];

  /* fill a bit */
  buf[0] = 0x80;
  Shash_sha1_update(pms, buf, 1);

  /* Decrement sha1_size1, sha1_size2 */
  if((pms->sha1_size1 -= Shash_sha1_bitS) == 0) {
    pms->sha1_size2--;
  }

  /* fill zeros */
  if(pms->sha1_count > sizeof(pms->sha1_buf) - 2 * sizeof(Shash_sha1_worD)) {
    memset(pms->sha1_buf + pms->sha1_count, 0, 
           sizeof(pms->sha1_buf) - pms->sha1_count);
    Shash_sha1_update_now(pms, pms->sha1_buf);
    pms->sha1_count = 0;
  }
  memset(pms->sha1_buf + pms->sha1_count, 0,
    sizeof(pms->sha1_buf) - pms->sha1_count - sizeof(Shash_sha1_worD) * 2);

  /* fill last length */
  Shash_sha1_unpackup(pms->sha1_buf + sizeof(pms->sha1_buf) 
                      - sizeof(Shash_sha1_worD) * 2, pms->sha1_size2);
  Shash_sha1_unpackup(pms->sha1_buf + sizeof(pms->sha1_buf)
                      - sizeof(Shash_sha1_worD) * 1, pms->sha1_size1);

  /* final update */
  Shash_sha1_update_now(pms, pms->sha1_buf);

  /* move hash value to output byte array */
  for(i = 0; i < sizeof(pms->sha1_h)/sizeof(Shash_sha1_worD); i++) {
    Shash_sha1_unpackup(output + i * sizeof(Shash_sha1_worD), pms->sha1_h[i]);
  }
}


int Shash_sha1(ctx,data,datalen,result,flag)
struct Shash_sha1_ctX **ctx;
                          /* *ctx has to be NULL or point to freeable memory */
unsigned char *data;
int datalen;
char result[20];
int flag;
/*
 bit0= allocate and init *ctx
 bit1= transfer ctx to result
 bit2= with bit 0 : clone new *ctx from (struct Shash_sha1_ctX *) data
 bit14= free *ctx after it has been transfered to result 
 bit15= free *ctx and exit
*/
{
 if(flag&(1<<15)) {
   if(*ctx!=NULL)
     Smem_freE((char *) *ctx);
   *ctx= NULL;
   return(1);
 }
 if(flag&1) {
   if(*ctx!=NULL)
     Smem_freE((char *) *ctx);
   *ctx= TSOB_FELD(struct Shash_sha1_ctX,1);
   if(*ctx==NULL)
     return(-1);
   Shash_sha1_init(*ctx,0);
   if(flag&4) {
     memcpy((char *) *ctx,data,sizeof(struct Shash_sha1_ctX));
     datalen= 0;
   }
 }
 if(*ctx==NULL)
   return(0);
 if(datalen>0)
   Shash_sha1_update(*ctx,data,datalen,0);
 if(flag&2) {
   Shash_sha1_finish(*ctx,result,0);
   if(flag&(1<<14)) {
     Smem_freE((char *) *ctx);
     *ctx= NULL;
   }
 }
 return(1);
}

/* ------------------------- end SHA-1 ----------------------- */


int Shash(ctx,data,datalen,result,mode,flag)
char **ctx;               /* *ctx has to be NULL or point to freeable memory */
unsigned char *data;
int datalen;
char result[20];
int mode; /* 1=md5 , 2=sha1 */
int flag;
/*
 bit0= allocate and init *ctx
 bit1= transfer ctx to result
 bit2= with bit 0 : clone new *ctx from (struct Shash_xyz_ctX *) data
 bit15= free *ctx
*/
{
 int ret;

 if(mode==1) {
   ret= Shash_md5(ctx,data,datalen,result,flag&0x8007);
 } else if(mode==2) {
   ret= Shash_sha1(ctx,data,datalen,result,flag&0x8007);
 } else {
   ret= -2;
 }
 return(ret);
}

